<?php
// -----------------------------------------------------------------------------
// $Id: search.php 26 2016-01-09 09:35:43Z Raoul $
//
// Copyright (C) 2016 Raoul Proença
// License: GNU GPL version 3 (see copying.txt file)
// Website: http://www.gnew.xyz/
// -----------------------------------------------------------------------------
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
// -----------------------------------------------------------------------------

include('./../includes/common.php');

page_header($lang['ARTICLES_SEARCH']);

if (isset($_GET['search']))
{
    // Clean and format string
    $pattern = array('`[ ]+`', '`[?]+`');
    $replace = array(' ', '?');
    $str = preg_replace($pattern, $replace, trim($_GET['search']));
    $str = utf8_entities(stripslashes($str));

    $search = '';
    $length = strlen($str);
    for ($i = 0; $i < $length; $i++)
    {
        // Valid search string: at least 3 characters long (see below)
        // and composed of letters (accented or not), digits or - and _ characters
        if (preg_match('`[a-z0-9_\-&#;]`i', $str{$i}))
        {
            $search .= $str{$i};
        }
        elseif ($str{$i} == ' ')
        {
            $search .= '%';
        }
        elseif ($str{$i} == '?')
        {
            $search .= '_';
        }
        else
        {
            $search .= '';
        }
    }

    // HTML entities
    preg_match_all('`&[^;\s]+;`', $search, $entities);
    $num_entities = count($entities[0]);
    $offset = 0;
    for ($i = 0; $i < $num_entities; $i++)
    {
        $offset += strlen($entities[0][$i]);
    }

    // Check string
    if (!empty($search) && (strlen($search) - $offset + $num_entities) >= 3)
    {
        if (empty($_GET['match']) || !preg_match('`^(article_subject|article_text)$`', $_GET['match']))
        {
            $_GET['match'] = 'article_subject';
        }
        $num_articles = 0;
        $template->set_file('search', 'articles/search.htpl');
        $template->set_block('search', 'ARTICLES_BLOCK', 'articles');
        $sql->query('SELECT t1.article_id, t1.category_id, t1.user_id, t1.article_subject, t1.article_text, t1.article_creation, t1.article_edition, t1.edition_id, t1.edition_name, t2.category_image, t2.category_name, t3.user_name
                     FROM ' . TABLE_ARTICLES . ' AS t1
                     INNER JOIN ' . TABLE_CATEGORIES . ' AS t2
                                 ON (t2.category_id = t1.category_id AND t2.category_level IN (\'1\', \'4\'))
                     INNER JOIN ' . TABLE_USERS . ' AS t3
                                 ON (t3.user_id = t1.user_id)
                     WHERE LOWER(t1.' . $_GET['match'] . ') LIKE \'%' . strtolower($search) . '%\'
                     ORDER BY t1.article_creation DESC
                     LIMIT ' . ($settings['search_limit'] + 1));
        while ($table_articles = $sql->fetch())
        {
            // Count articles
            $num_articles++;

            // Temporary tags to delimit search string - only elements outside HTML tags are highlighted
            $article_subject = '<!-- GNEW_TEMP -->' . $table_articles['article_subject'] . '<!-- GNEW_TEMP -->';
            $article_text = '<!-- GNEW_TEMP -->' . $table_articles['article_text'] . '<!-- GNEW_TEMP -->';

            $elements = explode('%', $search);
            $num_elements = count($elements);
            for ($i = 0; $i < $num_elements; $i++)
            {
                // Highlight elements
                if ($_GET['match'] == 'article_subject')
                {
                    $article_subject = preg_replace('`([^&#])(' . $elements[$i] . ')([^;])?(?=[^>]*<)`i', '\\1<span class="new">\\2</span>\\3', $article_subject);
                }
                if ($_GET['match'] == 'article_text')
                {
                    $article_text = preg_replace('`([^&#])(' . $elements[$i] . ')([^;])?(?=[^>]*<)`i', '\\1<span class="new">\\2</span>\\3', $article_text);
                }
            }

            // Remove temporary tags
            $article_subject = str_replace('<!-- GNEW_TEMP -->', '', $article_subject);
            $article_text = str_replace('<!-- GNEW_TEMP -->', '', $article_text);

            if ($table_articles['article_edition'])
            {
                $article_edited = sprintf($lang['EDITED_BY'], $table_articles['edition_id'], $table_articles['edition_name'], format_date($table_articles['article_edition']));
            }
            else
            {
                $article_edited = '';
            }
            $article_text = undo_escape_sequences($article_text);
            $article_text = cut_text($article_text, './../articles/read.php?article_id=' . $table_articles['article_id']);
            $template->set_var(array('ARTICLE_EDITED' => $article_edited,
                                     'ARTICLE_ID' => $table_articles['article_id'],
                                     'ARTICLE_POSTED' => sprintf($lang['POSTED_BY'], $table_articles['user_id'], $table_articles['user_name'], format_date($table_articles['article_creation'])),
                                     'ARTICLE_SUBJECT' => $article_subject,
                                     'ARTICLE_TEXT' => $article_text,
                                     'CATEGORY_ID' => $table_articles['category_id'],
                                     'CATEGORY_IMAGE' => $table_articles['category_image'],
                                     'CATEGORY_NAME' => $table_articles['category_name']));
            $template->parse('ARTICLES_BLOCK', 'articles', true);
            $articles_exist = true;
        }
        if ($num_articles > $settings['search_limit'])
        {
            error_template(sprintf($lang['ERROR_SEARCH_LIMIT'], $settings['search_limit']));
        }
        else
        {
            if (isset($articles_exist))
            {
                    $template->set_var('HEADER_SEARCH', sprintf($lang['HEADER_SEARCH'], $str, $num_articles));
                    $template->parse('search', null, false, array(TABLE_ARTICLES, TABLE_CATEGORIES, TABLE_USERS));
            }
            else
            {
                error_template($lang['SEARCH_HELP']);
            }
        }
    }
    else
    {
        error_template($lang['ERROR_BAD_SEARCH']);
    }
}
else
{
    error_template($lang['ERROR_NO_DATA']);
}

page_footer();

?>