## -----------------------------------------------------------------------------
## $Id: mysql.sql 26 2016-01-09 09:35:43Z Raoul $
##
## Copyright (C) 2016 Raoul Proença
## License: GNU GPL version 3 (see copying.txt file)
## Website: http://www.gnew.xyz/
## -----------------------------------------------------------------------------
## This program is free software: you can redistribute it and/or modify it under
## the terms of the GNU General Public License as published by the Free Software
## Foundation, either version 3 of the License, or (at your option) any later
## version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
## FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public License along with
## this program. If not, see <http://www.gnu.org/licenses/>.
## -----------------------------------------------------------------------------

# ANSWERS

CREATE TABLE gnew_answers (
    answer_id INTEGER NOT NULL AUTO_INCREMENT,
    question_id INTEGER NOT NULL DEFAULT '0',
    answer_text CHARACTER VARYING(1280) NOT NULL DEFAULT '',
    answer_votes INTEGER NOT NULL DEFAULT '0',
    PRIMARY KEY (answer_id)
);

CREATE INDEX answers_answer_id ON gnew_answers (answer_id);
CREATE INDEX answers_question_id ON gnew_answers (question_id);

# ARTICLES

CREATE TABLE gnew_articles (
    article_id INTEGER NOT NULL AUTO_INCREMENT,
    category_id INTEGER NOT NULL DEFAULT '0',
    user_id INTEGER NOT NULL DEFAULT '0',
    article_subject CHARACTER VARYING(1280) NOT NULL DEFAULT '',
    article_text MEDIUMTEXT NOT NULL DEFAULT '',
    article_creation INTEGER NOT NULL DEFAULT '0',
    article_edition INTEGER NOT NULL DEFAULT '0',
    edition_id INTEGER NOT NULL DEFAULT '0',
    edition_name CHARACTER VARYING(640) NOT NULL DEFAULT '',
    PRIMARY KEY (article_id)
);

CREATE INDEX articles_article_id ON gnew_articles (article_id);
CREATE INDEX articles_category_id ON gnew_articles (category_id);
CREATE INDEX articles_user_id ON gnew_articles (user_id);
CREATE INDEX articles_article_subject ON gnew_articles (article_subject);
CREATE INDEX articles_article_text ON gnew_articles (article_text(16));
CREATE INDEX articles_edition_id ON gnew_articles (edition_id);

# CATEGORIES

CREATE TABLE gnew_categories (
    category_id INTEGER NOT NULL AUTO_INCREMENT,
    category_description CHARACTER VARYING(1280) NOT NULL DEFAULT '',
    category_image CHARACTER VARYING(256) NOT NULL DEFAULT '',
    category_level CHARACTER(1) NOT NULL DEFAULT '0',
    category_name CHARACTER VARYING(640) NOT NULL DEFAULT '',
    category_articles INTEGER NOT NULL DEFAULT '0',
    category_news INTEGER NOT NULL DEFAULT '0',
    category_posts INTEGER NOT NULL DEFAULT '0',
    category_threads INTEGER NOT NULL DEFAULT '0',
    category_creation INTEGER NOT NULL DEFAULT '0',
    category_edition INTEGER NOT NULL DEFAULT '0',
    edition_id INTEGER NOT NULL DEFAULT '0',
    edition_name CHARACTER VARYING(640) NOT NULL DEFAULT '',
    PRIMARY KEY (category_id)
);

CREATE INDEX categories_category_id ON gnew_categories (category_id);
CREATE INDEX categories_category_level ON gnew_categories (category_level);
CREATE INDEX categories_category_name ON gnew_categories (category_name);
CREATE INDEX categories_edition_id ON gnew_categories (edition_id);

# COMMENTS

CREATE TABLE gnew_comments (
    comment_id INTEGER NOT NULL AUTO_INCREMENT,
    story_id INTEGER NOT NULL DEFAULT '0',
    reply_id INTEGER NOT NULL DEFAULT '0',
    news_id INTEGER NOT NULL DEFAULT '0',
    user_id INTEGER NOT NULL DEFAULT '0',
    comment_active CHARACTER(1) NOT NULL DEFAULT '0',
    comment_subject CHARACTER VARYING(1280) NOT NULL DEFAULT '',
    comment_text MEDIUMTEXT NOT NULL DEFAULT '',
    comment_creation INTEGER NOT NULL DEFAULT '0',
    comment_edition INTEGER NOT NULL DEFAULT '0',
    edition_id INTEGER NOT NULL DEFAULT '0',
    edition_name CHARACTER VARYING(640) NOT NULL DEFAULT '',
    PRIMARY KEY (comment_id)
);

CREATE INDEX comments_comment_id ON gnew_comments (comment_id);
CREATE INDEX comments_story_id ON gnew_comments (story_id);
CREATE INDEX comments_reply_id ON gnew_comments (reply_id);
CREATE INDEX comments_news_id ON gnew_comments (news_id);
CREATE INDEX comments_user_id ON gnew_comments (user_id);
CREATE INDEX comments_comment_active ON gnew_comments (comment_active);
CREATE INDEX comments_comment_subject ON gnew_comments (comment_subject);
CREATE INDEX comments_comment_text ON gnew_comments (comment_text(16));
CREATE INDEX comments_edition_id ON gnew_comments (edition_id);

# EMOTICONS

CREATE TABLE gnew_emoticons (
    emoticon_id INTEGER NOT NULL AUTO_INCREMENT,
    emoticon_code CHARACTER VARYING(32) NOT NULL DEFAULT '',
    emoticon_image CHARACTER VARYING(256) NOT NULL DEFAULT '',
    emoticon_creation INTEGER NOT NULL DEFAULT '0',
    emoticon_edition INTEGER NOT NULL DEFAULT '0',
    edition_id INTEGER NOT NULL DEFAULT '0',
    edition_name CHARACTER VARYING(640) NOT NULL DEFAULT '',
    PRIMARY KEY (emoticon_id)
);

CREATE INDEX emoticons_emoticon_id ON gnew_emoticons (emoticon_id);
CREATE INDEX emoticons_emoticon_code ON gnew_emoticons (emoticon_code);
CREATE INDEX emoticons_edition_id ON gnew_emoticons (edition_id);

# NEWS

CREATE TABLE gnew_news (
    news_id INTEGER NOT NULL AUTO_INCREMENT,
    category_id INTEGER NOT NULL DEFAULT '0',
    user_id INTEGER NOT NULL DEFAULT '0',
    news_active CHARACTER(1) NOT NULL DEFAULT '0',
    news_comments INTEGER NOT NULL DEFAULT '0',
    news_month CHARACTER(2) NOT NULL DEFAULT '01',
    news_source TEXT NOT NULL DEFAULT '',
    news_subject CHARACTER VARYING(1280) NOT NULL DEFAULT '',
    news_text MEDIUMTEXT NOT NULL DEFAULT '',
    news_year CHARACTER(4) NOT NULL DEFAULT '1970',
    news_creation INTEGER NOT NULL DEFAULT '0',
    news_edition INTEGER NOT NULL DEFAULT '0',
    edition_id INTEGER NOT NULL DEFAULT '0',
    edition_name CHARACTER VARYING(640) NOT NULL DEFAULT '',
    PRIMARY KEY (news_id)
);

CREATE INDEX news_news_id ON gnew_news (news_id);
CREATE INDEX news_category_id ON gnew_news (category_id);
CREATE INDEX news_user_id ON gnew_news (user_id);
CREATE INDEX news_news_active ON gnew_news (news_active);
CREATE INDEX news_news_month ON gnew_news (news_month);
CREATE INDEX news_news_subject ON gnew_news (news_subject);
CREATE INDEX news_news_text ON gnew_news (news_text(16));
CREATE INDEX news_news_year ON gnew_news (news_year);
CREATE INDEX news_edition_id ON gnew_news (edition_id);

# POSTS

CREATE TABLE gnew_posts (
    post_id INTEGER NOT NULL AUTO_INCREMENT,
    thread_id INTEGER NOT NULL DEFAULT '0',
    category_id INTEGER NOT NULL DEFAULT '0',
    user_id INTEGER NOT NULL DEFAULT '0',
    post_active CHARACTER(1) NOT NULL DEFAULT '0',
    post_sticky CHARACTER(1) NOT NULL DEFAULT '0',
    post_subject CHARACTER VARYING(1280) NOT NULL DEFAULT '',
    post_text MEDIUMTEXT NOT NULL DEFAULT '',
    post_creation INTEGER NOT NULL DEFAULT '0',
    post_edition INTEGER NOT NULL DEFAULT '0',
    edition_id INTEGER NOT NULL DEFAULT '0',
    edition_name CHARACTER VARYING(640) NOT NULL DEFAULT '',
    PRIMARY KEY (post_id)
);

CREATE INDEX posts_post_id ON gnew_posts (post_id);
CREATE INDEX posts_thread_id ON gnew_posts (thread_id);
CREATE INDEX posts_category_id ON gnew_posts (category_id);
CREATE INDEX posts_user_id ON gnew_posts (user_id);
CREATE INDEX posts_post_subject ON gnew_posts (post_subject);
CREATE INDEX posts_post_text ON gnew_posts (post_text(16));
CREATE INDEX posts_post_creation ON gnew_posts (post_creation);
CREATE INDEX posts_edition_id ON gnew_posts (edition_id);

# QUESTIONS

CREATE TABLE gnew_questions (
    question_id INTEGER NOT NULL AUTO_INCREMENT,
    user_id INTEGER NOT NULL DEFAULT '0',
    question_answers INTEGER NOT NULL DEFAULT '0',
    question_comment TEXT NOT NULL DEFAULT '',
    question_text CHARACTER VARYING(1280) NOT NULL DEFAULT '',
    question_votes INTEGER NOT NULL DEFAULT '0',
    question_creation INTEGER NOT NULL DEFAULT '0',
    question_edition INTEGER NOT NULL DEFAULT '0',
    edition_id INTEGER NOT NULL DEFAULT '0',
    edition_name CHARACTER VARYING(640) NOT NULL DEFAULT '',
    PRIMARY KEY (question_id)
);

CREATE INDEX questions_question_id ON gnew_questions (question_id);
CREATE INDEX questions_user_id ON gnew_questions (user_id);
CREATE INDEX questions_edition_id ON gnew_questions (edition_id);

# SESSIONS

CREATE TABLE gnew_sessions (
    session_id CHARACTER VARYING(40) NOT NULL DEFAULT '',
    session_data CHARACTER VARYING(65) NOT NULL DEFAULT '',
    session_expiry INTEGER NOT NULL DEFAULT '0'
);

CREATE INDEX sessions_session_id ON gnew_sessions (session_id);
CREATE INDEX sessions_session_expiry ON gnew_sessions (session_expiry);

# SETTINGS

CREATE TABLE gnew_settings (
    accounts_expiry CHARACTER VARYING(3) NOT NULL DEFAULT '30',
    allow_emoticons CHARACTER(1) NOT NULL DEFAULT '1',
    allow_html CHARACTER(1) NOT NULL DEFAULT '0',
    articles_order CHARACTER VARYING(4) NOT NULL DEFAULT 'DESC',
    articles_per_page CHARACTER VARYING(2) NOT NULL DEFAULT '20',
    articles_sorting CHARACTER VARYING(16) NOT NULL DEFAULT 'article_creation',
    categories_order CHARACTER VARYING(4) NOT NULL DEFAULT 'DESC',
    categories_per_page CHARACTER VARYING(2) NOT NULL DEFAULT '20',
    categories_sorting CHARACTER VARYING(17) NOT NULL DEFAULT 'category_creation',
    comments_interval CHARACTER VARYING(2) NOT NULL DEFAULT '30',
    comments_order CHARACTER VARYING(4) NOT NULL DEFAULT 'ASC',
    comments_per_page CHARACTER VARYING(2) NOT NULL DEFAULT '20',
    comments_sorting CHARACTER VARYING(16) NOT NULL DEFAULT 'comment_creation',
    cookies_expiry CHARACTER VARYING(3) NOT NULL DEFAULT '30',
    date_format CHARACTER VARYING(32) NOT NULL DEFAULT 'D, M jS Y, g:i a',
    date_offset CHARACTER VARYING(3) NOT NULL DEFAULT '0',
    headlines_per_feed CHARACTER VARYING(2) NOT NULL DEFAULT '10',
    language CHARACTER VARYING(32) NOT NULL DEFAULT 'english',
    language_unique CHARACTER(1) NOT NULL DEFAULT '0',
    max_avatars_pixels CHARACTER VARYING(3) NOT NULL DEFAULT '64',
    max_submissions_per_user CHARACTER VARYING(2) NOT NULL DEFAULT '3',
    max_texts_length CHARACTER VARYING(5) NOT NULL DEFAULT '1000',
    min_passwords_length CHARACTER VARYING(2) NOT NULL DEFAULT '6',
    min_usernames_length CHARACTER VARYING(2) NOT NULL DEFAULT '3',
    news_commenting CHARACTER(1) NOT NULL DEFAULT '1',
    news_order CHARACTER VARYING(4) NOT NULL DEFAULT 'DESC',
    news_per_page CHARACTER VARYING(2) NOT NULL DEFAULT '10',
    news_sending CHARACTER(1) NOT NULL DEFAULT '1',
    news_sorting CHARACTER VARYING(13) NOT NULL DEFAULT 'news_creation',
    news_submitting CHARACTER(1) NOT NULL DEFAULT '1',
    pages_limit CHARACTER VARYING(2) NOT NULL DEFAULT '10',
    posts_interval CHARACTER VARYING(2) NOT NULL DEFAULT '30',
    posts_order CHARACTER VARYING(4) NOT NULL DEFAULT 'ASC',
    posts_per_page CHARACTER VARYING(2) NOT NULL DEFAULT '20',
    posts_sorting CHARACTER VARYING(13) NOT NULL DEFAULT 'post_creation',
    questions_order CHARACTER VARYING(4) NOT NULL DEFAULT 'DESC',
    questions_per_page CHARACTER VARYING(2) NOT NULL DEFAULT '20',
    questions_sorting CHARACTER VARYING(17) NOT NULL DEFAULT 'question_creation',
    search_limit CHARACTER VARYING(2) NOT NULL DEFAULT '20',
    sender_email CHARACTER VARYING(256) NOT NULL DEFAULT '',
    sender_name CHARACTER VARYING(640) NOT NULL DEFAULT '',
    site_name CHARACTER VARYING(640) NOT NULL DEFAULT '',
    site_url CHARACTER VARYING(256) NOT NULL DEFAULT '',
    stories_expiry CHARACTER VARYING(3) NOT NULL DEFAULT '730',
    stories_per_page CHARACTER VARYING(2) NOT NULL DEFAULT '10',
    template CHARACTER VARYING(32) NOT NULL DEFAULT 'clean',
    template_unique CHARACTER(1) NOT NULL DEFAULT '0',
    threads_expiry CHARACTER VARYING(3) NOT NULL DEFAULT '730',
    threads_per_page CHARACTER VARYING(2) NOT NULL DEFAULT '20',
    users_order CHARACTER VARYING(4) NOT NULL DEFAULT 'DESC',
    users_per_page CHARACTER VARYING(2) NOT NULL DEFAULT '20',
    users_registering CHARACTER(1) NOT NULL DEFAULT '1',
    users_sorting CHARACTER VARYING(13) NOT NULL DEFAULT 'user_creation',
    votes_interval CHARACTER VARYING(3) NOT NULL DEFAULT '30'
);

# USERS

CREATE TABLE gnew_users (
    user_id INTEGER NOT NULL AUTO_INCREMENT,
    user_articles INTEGER NOT NULL DEFAULT '0',
    user_avatar CHARACTER VARYING(256) NOT NULL DEFAULT './../images/avatars/empty.png',
    user_comments INTEGER NOT NULL DEFAULT '0',
    user_date_format CHARACTER VARYING(32) NOT NULL DEFAULT 'D, M jS Y, g:i a',
    user_date_offset CHARACTER VARYING(3) NOT NULL DEFAULT '0',
    user_day CHARACTER VARYING(2) NOT NULL DEFAULT '0',
    user_email CHARACTER VARYING(256) NOT NULL DEFAULT '',
    user_ip CHARACTER VARYING(39) NOT NULL DEFAULT '',
    user_key CHARACTER VARYING(40) NOT NULL DEFAULT '',
    user_language CHARACTER VARYING(32) NOT NULL DEFAULT 'english',
    user_last_visit INTEGER NOT NULL DEFAULT '0',
    user_level CHARACTER(1) NOT NULL DEFAULT '1',
    user_location CHARACTER VARYING(1280) NOT NULL DEFAULT '',
    user_month CHARACTER VARYING(2) NOT NULL DEFAULT '0',
    user_name CHARACTER VARYING(640) NOT NULL DEFAULT '',
    user_news INTEGER NOT NULL DEFAULT '0',
    user_occupation CHARACTER VARYING(1280) NOT NULL DEFAULT '',
    user_password CHARACTER(40) NOT NULL DEFAULT '',
    user_polls INTEGER NOT NULL DEFAULT '0',
    user_posts INTEGER NOT NULL DEFAULT '0',
    user_show_email CHARACTER(1) NOT NULL DEFAULT '0',
    user_signature CHARACTER VARYING(1280) NOT NULL DEFAULT '',
    user_submissions INTEGER NOT NULL DEFAULT '0',
    user_template CHARACTER VARYING(32) NOT NULL DEFAULT 'clean',
    user_votes INTEGER NOT NULL DEFAULT '0',
    user_website CHARACTER VARYING(256) NOT NULL DEFAULT '',
    user_year CHARACTER VARYING(4) NOT NULL DEFAULT '0',
    user_creation INTEGER NOT NULL DEFAULT '0',
    user_edition INTEGER NOT NULL DEFAULT '0',
    edition_id INTEGER NOT NULL DEFAULT '0',
    edition_name CHARACTER VARYING(640) NOT NULL DEFAULT '',
    PRIMARY KEY (user_id)
);

CREATE INDEX users_user_id ON gnew_users (user_id);
CREATE INDEX users_user_email ON gnew_users (user_email);
CREATE INDEX users_user_key ON gnew_users (user_key);
CREATE INDEX users_user_level ON gnew_users (user_level);
CREATE INDEX users_user_name ON gnew_users (user_name);
CREATE INDEX users_user_password ON gnew_users (user_password);
CREATE INDEX users_user_creation ON gnew_users (user_creation);
CREATE INDEX users_edition_id ON gnew_users (edition_id);

# VOTES

CREATE TABLE gnew_votes (
    vote_id INTEGER NOT NULL AUTO_INCREMENT,
    question_id INTEGER NOT NULL DEFAULT '0',
    answer_id INTEGER NOT NULL DEFAULT '0',
    user_id INTEGER NOT NULL DEFAULT '0',
    vote_creation INTEGER NOT NULL DEFAULT '0',
    PRIMARY KEY (vote_id)
);

CREATE INDEX votes_question_id ON gnew_votes (question_id);
CREATE INDEX votes_answer_id ON gnew_votes (answer_id);
CREATE INDEX votes_user_id ON gnew_votes (user_id);

# END