<?php
// -----------------------------------------------------------------------------
// $Id: search.php 26 2016-01-09 09:35:43Z Raoul $
//
// Copyright (C) 2016 Raoul Proença
// License: GNU GPL version 3 (see copying.txt file)
// Website: http://www.gnew.xyz/
// -----------------------------------------------------------------------------
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program. If not, see <http://www.gnu.org/licenses/>.
// -----------------------------------------------------------------------------

include('./../includes/common.php');

page_header($lang['NEWS_SEARCH']);

if (isset($_GET['search']))
{
    // Clean and format string
    $pattern = array('`[ ]+`', '`[?]+`');
    $replace = array(' ', '?');
    $str = preg_replace($pattern, $replace, trim($_GET['search']));
    $str = utf8_entities(stripslashes($str));

    $search = '';
    $length = strlen($str);
    for ($i = 0; $i < $length; $i++)
    {
        // Valid search string: at least 3 characters long (see below)
        // and composed of letters (accented or not), digits or - and _ characters
        if (preg_match('`[a-z0-9_\-&#;]`i', $str{$i}))
        {
            $search .= $str{$i};
        }
        elseif ($str{$i} == ' ')
        {
            $search .= '%';
        }
        elseif ($str{$i} == '?')
        {
            $search .= '_';
        }
        else
        {
            $search .= '';
        }
    }

    // HTML entities
    preg_match_all('`&[^;\s]+;`', $search, $entities);
    $num_entities = count($entities[0]);
    $offset = 0;
    for ($i = 0; $i < $num_entities; $i++)
    {
        $offset += strlen($entities[0][$i]);
    }

    // Check string
    if (!empty($search) && (strlen($search) - $offset + $num_entities) >= 3)
    {
        if (empty($_GET['match']) || !preg_match('`^(news_subject|news_text)$`', $_GET['match']))
        {
            $_GET['match'] = 'news_subject';
        }
        $num_news = 0;
        $template->set_file('search', 'news/search.htpl');
        $template->set_block('search', 'NEWS_BLOCK', 'news');
        $sql->query('SELECT t1.news_id, t1.category_id, t1.user_id, t1.news_comments, t1.news_source, t1.news_subject, t1.news_text, t1.news_creation, t1.news_edition, t1.edition_id, t1.edition_name, t2.category_image, t2.category_name, t3.user_name
                     FROM ' . TABLE_NEWS . ' AS t1
                     INNER JOIN ' . TABLE_CATEGORIES . ' AS t2
                                 ON (t2.category_id = t1.category_id AND t2.category_level IN (\'2\', \'4\'))
                     INNER JOIN ' . TABLE_USERS . ' AS t3
                                 ON (t3.user_id = t1.user_id)
                     WHERE t1.news_active = \'1\'
                     AND LOWER(t1.' . $_GET['match'] . ') LIKE \'%' . strtolower($search) . '%\'
                     ORDER BY t1.news_creation DESC
                     LIMIT ' . ($settings['search_limit'] + 1));
        while ($table_news = $sql->fetch())
        {
            // Count news
            $num_news++;

            // Temporary tags to delimit search string - only elements outside HTML tags are highlighted
            $news_subject = '<!-- GNEW_TEMP -->' . $table_news['news_subject'] . '<!-- GNEW_TEMP -->';
            $news_text = '<!-- GNEW_TEMP -->' . $table_news['news_text'] . '<!-- GNEW_TEMP -->';

            $elements = explode('%', $search);
            $num_elements = count($elements);
            for ($i = 0; $i < $num_elements; $i++)
            {
                // Highlight elements
                if ($_GET['match'] == 'news_subject')
                {
                    $news_subject = preg_replace('`([^&#])(' . $elements[$i] . ')([^;])?(?=[^>]*<)`i', '\\1<span class="new">\\2</span>\\3', $news_subject);
                }
                if ($_GET['match'] == 'news_text')
                {
                    $news_text = preg_replace('`([^&#])(' . $elements[$i] . ')([^;])?(?=[^>]*<)`i', '\\1<span class="new">\\2</span>\\3', $news_text);
                }
            }

            // Remove temporary tags
            $news_subject = str_replace('<!-- GNEW_TEMP -->', '', $news_subject);
            $news_text = str_replace('<!-- GNEW_TEMP -->', '', $news_text);

            if ($table_news['news_edition'])
            {
                $news_edited = sprintf($lang['EDITED_BY'], $table_news['edition_id'], $table_news['edition_name'], format_date($table_news['news_edition']));
            }
            else
            {
                $news_edited = '';
            }
            if ($table_news['news_source'])
            {
                $news_source = undo_escape_sequences($table_news['news_source']);
                $news_source = sprintf($lang['SOURCE'], $news_source);
            }
            else
            {
                $news_source = '<!-- empty -->';
            }
            $news_text = undo_escape_sequences($news_text);
            $news_text = cut_text($news_text, './../news/index.php?news_id=' . $table_news['news_id']);
            $template->set_var(array('CATEGORY_ID' => $table_news['category_id'],
                                     'CATEGORY_IMAGE' => $table_news['category_image'],
                                     'CATEGORY_NAME' => $table_news['category_name'],
                                     'NEWS_COMMENTS' => $table_news['news_comments'],
                                     'NEWS_EDITED' => $news_edited,
                                     'NEWS_ID' => $table_news['news_id'],
                                     'NEWS_POSTED' => sprintf($lang['POSTED_BY'], $table_news['user_id'], $table_news['user_name'], format_date($table_news['news_creation'])),
                                     'NEWS_SOURCE' => $news_source,
                                     'NEWS_SUBJECT' => $news_subject,
                                     'NEWS_TEXT' => $news_text));
            $template->parse('NEWS_BLOCK', 'news', true);
            $news_exist = true;
        }
        if ($num_news > $settings['search_limit'])
        {
            error_template(sprintf($lang['ERROR_SEARCH_LIMIT'], $settings['search_limit']));
        }
        else
        {
            if (isset($news_exist))
            {
                    $template->set_var('HEADER_SEARCH', sprintf($lang['HEADER_SEARCH'], $str, $num_news));
                    $template->parse('search', null, false, array(TABLE_CATEGORIES, TABLE_NEWS, TABLE_USERS));
            }
            else
            {
                error_template($lang['SEARCH_HELP']);
            }
        }
    }
    else
    {
        error_template($lang['ERROR_BAD_SEARCH']);
    }
}
else
{
    error_template($lang['ERROR_NO_DATA']);
}

page_footer();

?>